import { atom } from 'jotai';
import { createClient } from '@urql/core';
import { pipe, subscribe } from 'wonka';

const DEFAULT_URL = typeof process === "object" && process.env.JOTAI_URQL_DEFAULT_URL || "/graphql";
const clientAtom = atom(createClient({ url: DEFAULT_URL }));

var __defProp = Object.defineProperty;
var __getOwnPropSymbols = Object.getOwnPropertySymbols;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __propIsEnum = Object.prototype.propertyIsEnumerable;
var __defNormalProp = (obj, key, value) => key in obj ? __defProp(obj, key, { enumerable: true, configurable: true, writable: true, value }) : obj[key] = value;
var __spreadValues = (a, b) => {
  for (var prop in b || (b = {}))
    if (__hasOwnProp.call(b, prop))
      __defNormalProp(a, prop, b[prop]);
  if (__getOwnPropSymbols)
    for (var prop of __getOwnPropSymbols(b)) {
      if (__propIsEnum.call(b, prop))
        __defNormalProp(a, prop, b[prop]);
    }
  return a;
};
function atomWithQuery(createQueryArgs, getClient = (get) => get(clientAtom)) {
  const queryResultAtom = atom((get) => {
    const client = getClient(get);
    const args = createQueryArgs(get);
    let resolve = null;
    const resultAtom = atom(new Promise((r) => {
      resolve = r;
    }));
    resultAtom.scope = queryAtom.scope;
    let setResult = () => {
      throw new Error("setting result without mount");
    };
    const listener = (result) => {
      if (resolve) {
        resolve(result);
        resolve = null;
      } else {
        setResult(result);
      }
    };
    client.query(args.query, args.variables, __spreadValues({
      requestPolicy: args.requestPolicy
    }, args.context)).toPromise().then(listener).catch(() => {
    });
    resultAtom.onMount = (update) => {
      setResult = update;
      const subscription = pipe(client.query(args.query, args.variables, __spreadValues({
        requestPolicy: args.requestPolicy
      }, args.context)), subscribe(listener));
      return () => subscription.unsubscribe();
    };
    return { resultAtom, args };
  });
  const queryAtom = atom((get) => {
    queryResultAtom.scope = queryAtom.scope;
    const { resultAtom } = get(queryResultAtom);
    return get(resultAtom);
  });
  return queryAtom;
}

function atomWithMutation(createQuery, getClient = (get) => get(clientAtom)) {
  const operationResultAtom = atom(new Promise(() => {
  }));
  const queryResultAtom = atom((get) => {
    operationResultAtom.scope = queryResultAtom.scope;
    return get(operationResultAtom);
  }, (get, set, action) => {
    operationResultAtom.scope = queryResultAtom.scope;
    set(operationResultAtom, new Promise(() => {
    }));
    const client = getClient(get);
    const query = createQuery(get);
    client.mutation(query, action.variables, action.context).toPromise().then((result) => {
      var _a;
      set(operationResultAtom, result);
      (_a = action.callback) == null ? void 0 : _a.call(action, result);
    }).catch(() => {
    });
  });
  return queryResultAtom;
}

function atomWithSubscription(createSubscriptionArgs, getClient = (get) => get(clientAtom)) {
  const queryResultAtom = atom((get) => {
    const client = getClient(get);
    const args = createSubscriptionArgs(get);
    let resolve = null;
    const resultAtom = atom(new Promise((r) => {
      resolve = r;
    }));
    resultAtom.scope = queryAtom.scope;
    let setResult = () => {
      throw new Error("setting result without mount");
    };
    const listener = (result) => {
      if (resolve) {
        resolve(result);
        resolve = null;
      } else {
        setResult(result);
      }
    };
    const subscriptionInRender = pipe(client.subscription(args.query, args.variables, args.context), subscribe(listener));
    let timer = setTimeout(() => {
      timer = null;
      subscriptionInRender.unsubscribe();
    }, 1e3);
    resultAtom.onMount = (update) => {
      setResult = update;
      let subscription;
      if (timer) {
        clearTimeout(timer);
        subscription = subscriptionInRender;
      } else {
        subscription = pipe(client.subscription(args.query, args.variables, args.context), subscribe(listener));
      }
      return () => subscription.unsubscribe();
    };
    return { resultAtom, args };
  });
  const queryAtom = atom((get) => {
    queryResultAtom.scope = queryAtom.scope;
    const { resultAtom } = get(queryResultAtom);
    return get(resultAtom);
  });
  return queryAtom;
}

export { atomWithMutation, atomWithQuery, atomWithSubscription, clientAtom };
