import { atom } from 'jotai';

function atomWithStore(store) {
  const baseAtom = atom(store.getState());
  baseAtom.onMount = (setValue) => {
    const callback = () => {
      setValue(store.getState());
    };
    const unsub = store.subscribe(callback);
    callback();
    return unsub;
  };
  const derivedAtom = atom((get) => {
    baseAtom.scope = derivedAtom.scope;
    return get(baseAtom);
  }, (get, _set, update) => {
    baseAtom.scope = derivedAtom.scope;
    const newState = typeof update === "function" ? update(get(baseAtom)) : update;
    store.setState(newState, true);
  });
  return derivedAtom;
}

export { atomWithStore };
