'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

var jotai = require('jotai');
var core = require('@urql/core');
var wonka = require('wonka');

var DEFAULT_URL = typeof process === 'object' && process.env.JOTAI_URQL_DEFAULT_URL || '/graphql';
var clientAtom = jotai.atom(core.createClient({
  url: DEFAULT_URL
}));

function _extends() {
  _extends = Object.assign || function (target) {
    for (var i = 1; i < arguments.length; i++) {
      var source = arguments[i];

      for (var key in source) {
        if (Object.prototype.hasOwnProperty.call(source, key)) {
          target[key] = source[key];
        }
      }
    }

    return target;
  };

  return _extends.apply(this, arguments);
}

function atomWithQuery(createQueryArgs, getClient) {
  if (getClient === void 0) {
    getClient = function getClient(get) {
      return get(clientAtom);
    };
  }

  var queryResultAtom = jotai.atom(function (get) {
    var client = getClient(get);
    var args = createQueryArgs(get);
    var resolve = null;
    var resultAtom = jotai.atom(new Promise(function (r) {
      resolve = r;
    }));
    resultAtom.scope = queryAtom.scope;

    var setResult = function setResult() {
      throw new Error('setting result without mount');
    };

    var listener = function listener(result) {
      if (resolve) {
        resolve(result);
        resolve = null;
      } else {
        setResult(result);
      }
    };

    client.query(args.query, args.variables, _extends({
      requestPolicy: args.requestPolicy
    }, args.context)).toPromise().then(listener).catch(function () {});

    resultAtom.onMount = function (update) {
      setResult = update;
      var subscription = wonka.pipe(client.query(args.query, args.variables, _extends({
        requestPolicy: args.requestPolicy
      }, args.context)), wonka.subscribe(listener));
      return function () {
        return subscription.unsubscribe();
      };
    };

    return {
      resultAtom: resultAtom,
      args: args
    };
  });
  var queryAtom = jotai.atom(function (get) {
    queryResultAtom.scope = queryAtom.scope;

    var _get = get(queryResultAtom),
        resultAtom = _get.resultAtom;

    return get(resultAtom);
  });
  return queryAtom;
}

function atomWithMutation(createQuery, getClient) {
  if (getClient === void 0) {
    getClient = function getClient(get) {
      return get(clientAtom);
    };
  }

  var operationResultAtom = jotai.atom(new Promise(function () {}));
  var queryResultAtom = jotai.atom(function (get) {
    operationResultAtom.scope = queryResultAtom.scope;
    return get(operationResultAtom);
  }, function (get, set, action) {
    operationResultAtom.scope = queryResultAtom.scope;
    set(operationResultAtom, new Promise(function () {}));
    var client = getClient(get);
    var query = createQuery(get);
    client.mutation(query, action.variables, action.context).toPromise().then(function (result) {
      set(operationResultAtom, result);
      action.callback == null ? void 0 : action.callback(result);
    }).catch(function () {});
  });
  return queryResultAtom;
}

function atomWithSubscription(createSubscriptionArgs, getClient) {
  if (getClient === void 0) {
    getClient = function getClient(get) {
      return get(clientAtom);
    };
  }

  var queryResultAtom = jotai.atom(function (get) {
    var client = getClient(get);
    var args = createSubscriptionArgs(get);
    var resolve = null;
    var resultAtom = jotai.atom(new Promise(function (r) {
      resolve = r;
    }));
    resultAtom.scope = queryAtom.scope;

    var setResult = function setResult() {
      throw new Error('setting result without mount');
    };

    var listener = function listener(result) {
      if (resolve) {
        resolve(result);
        resolve = null;
      } else {
        setResult(result);
      }
    };

    var subscriptionInRender = wonka.pipe(client.subscription(args.query, args.variables, args.context), wonka.subscribe(listener));
    var timer = setTimeout(function () {
      timer = null;
      subscriptionInRender.unsubscribe();
    }, 1000);

    resultAtom.onMount = function (update) {
      setResult = update;
      var subscription;

      if (timer) {
        clearTimeout(timer);
        subscription = subscriptionInRender;
      } else {
        subscription = wonka.pipe(client.subscription(args.query, args.variables, args.context), wonka.subscribe(listener));
      }

      return function () {
        return subscription.unsubscribe();
      };
    };

    return {
      resultAtom: resultAtom,
      args: args
    };
  });
  var queryAtom = jotai.atom(function (get) {
    queryResultAtom.scope = queryAtom.scope;

    var _get = get(queryResultAtom),
        resultAtom = _get.resultAtom;

    return get(resultAtom);
  });
  return queryAtom;
}

exports.atomWithMutation = atomWithMutation;
exports.atomWithQuery = atomWithQuery;
exports.atomWithSubscription = atomWithSubscription;
exports.clientAtom = clientAtom;
